@@ -11,10 +11,13 @@
 
 namespace Symfony\Bundle\TwigBundle\Controller;
 
-use Symfony\Component\HttpKernel\Exception\FlattenException;
+use Symfony\Component\Debug\Exception\FlattenException;
 use Symfony\Component\HttpKernel\Log\DebugLoggerInterface;
 use Symfony\Component\HttpFoundation\Request;
 use Symfony\Component\HttpFoundation\Response;
+use Twig\Environment;
+use Twig\Error\LoaderError;
+use Twig\Loader\ExistsLoaderInterface;
 
 /**
  * ExceptionController renders error or exception pages for a given
@@ -26,13 +29,13 @@ use Symfony\Component\HttpFoundation\Response;
 class ExceptionController
 {
     protected $twig;
-
-    /**
-     * @var bool Show error (false) or exception (true) pages by default.
-     */
     protected $debug;
 
-    public function __construct(\Twig_Environment $twig, $debug)
+    /**
+     * @param Environment $twig
+     * @param bool        $debug Show error (false) or exception (true) pages by default
+     */
+    public function __construct(Environment $twig, $debug)
     {
         $this->twig = $twig;
         $this->debug = $debug;
@@ -45,10 +48,6 @@ class ExceptionController
      * the exception page (when true). If it is not present, the "debug" value passed into the constructor will
      * be used.
      *
-     * @param Request              $request   The request
-     * @param FlattenException     $exception A FlattenException instance
-     * @param DebugLoggerInterface $logger    A DebugLoggerInterface instance
-     *
      * @return Response
      *
      * @throws \InvalidArgumentException When the exception template does not exist
@@ -69,7 +68,7 @@ class ExceptionController
                 'logger' => $logger,
                 'currentContent' => $currentContent,
             )
-        ));
+        ), 200, array('Content-Type' => $request->getMimeType($request->getRequestFormat()) ?: 'text/html'));
     }
 
     /**
@@ -129,15 +128,15 @@ class ExceptionController
         $template = (string) $template;
 
         $loader = $this->twig->getLoader();
-        if ($loader instanceof \Twig_ExistsLoaderInterface) {
+        if ($loader instanceof ExistsLoaderInterface || method_exists($loader, 'exists')) {
             return $loader->exists($template);
         }
 
         try {
-            $loader->getSource($template);
+            $loader->getSourceContext($template)->getCode();
 
             return true;
-        } catch (\Twig_Error_Loader $e) {
+        } catch (LoaderError $e) {
         }
 
         return false;
