@@ -18,17 +18,16 @@ use Symfony\Component\Form\FormInterface;
 use Symfony\Component\Form\Extension\Validator\Constraints\Form;
 use Symfony\Component\Form\Extension\Validator\Constraints\FormValidator;
 use Symfony\Component\Form\SubmitButtonBuilder;
+use Symfony\Component\Validator\Constraints\GroupSequence;
 use Symfony\Component\Validator\Constraints\NotNull;
 use Symfony\Component\Validator\Constraints\NotBlank;
 use Symfony\Component\Validator\Constraints\Valid;
-use Symfony\Component\Validator\ExecutionContextInterface;
-use Symfony\Component\Validator\Tests\Constraints\AbstractConstraintValidatorTest;
-use Symfony\Component\Validator\Validation;
+use Symfony\Component\Validator\Test\ConstraintValidatorTestCase;
 
 /**
  * @author Bernhard Schussek <bschussek@gmail.com>
  */
-class FormValidatorTest extends AbstractConstraintValidatorTest
+class FormValidatorTest extends ConstraintValidatorTestCase
 {
     /**
      * @var \PHPUnit_Framework_MockObject_MockObject
@@ -47,21 +46,13 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     protected function setUp()
     {
-        $this->dispatcher = $this->getMock('Symfony\Component\EventDispatcher\EventDispatcherInterface');
-        $this->factory = $this->getMock('Symfony\Component\Form\FormFactoryInterface');
-        $this->serverParams = $this->getMock(
-            'Symfony\Component\Form\Extension\Validator\Util\ServerParams',
-            array('getNormalizedIniPostMaxSize', 'getContentLength')
-        );
+        $this->dispatcher = $this->getMockBuilder('Symfony\Component\EventDispatcher\EventDispatcherInterface')->getMock();
+        $this->factory = $this->getMockBuilder('Symfony\Component\Form\FormFactoryInterface')->getMock();
+        $this->serverParams = $this->getMockBuilder('Symfony\Component\Form\Extension\Validator\Util\ServerParams')->setMethods(array('getNormalizedIniPostMaxSize', 'getContentLength'))->getMock();
 
         parent::setUp();
     }
 
-    protected function getApiVersion()
-    {
-        return Validation::API_VERSION_2_5;
-    }
-
     protected function createValidator()
     {
         return new FormValidator($this->serverParams);
@@ -69,14 +60,13 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testValidate()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $options = array('validation_groups' => array('group1', 'group2'));
         $form = $this->getBuilder('name', '\stdClass', $options)
             ->setData($object)
             ->getForm();
 
-        $this->expectValidateAt(0, 'data', $object, 'group1');
-        $this->expectValidateAt(1, 'data', $object, 'group2');
+        $this->expectValidateAt(0, 'data', $object, array('group1', 'group2'));
 
         $this->validator->validate($form, new Form());
 
@@ -85,7 +75,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testValidateConstraints()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $constraint1 = new NotNull(array('groups' => array('group1', 'group2')));
         $constraint2 = new NotBlank(array('groups' => 'group2'));
 
@@ -98,43 +88,20 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
             ->getForm();
 
         // First default constraints
-        $this->expectValidateAt(0, 'data', $object, 'group1');
-        $this->expectValidateAt(1, 'data', $object, 'group2');
+        $this->expectValidateAt(0, 'data', $object, array('group1', 'group2'));
 
         // Then custom constraints
-        $this->expectValidateValueAt(2, 'data', $object, $constraint1, 'group1');
-        $this->expectValidateValueAt(3, 'data', $object, $constraint2, 'group2');
+        $this->expectValidateValueAt(1, 'data', $object, $constraint1, 'group1');
+        $this->expectValidateValueAt(2, 'data', $object, $constraint2, 'group2');
 
         $this->validator->validate($form, new Form());
 
         $this->assertNoViolation();
     }
 
-    public function testValidateIfParentWithCascadeValidation()
+    public function testValidateChildIfValidConstraint()
     {
-        $object = $this->getMock('\stdClass');
-
-        $parent = $this->getBuilder('parent', null, array('cascade_validation' => true))
-            ->setCompound(true)
-            ->setDataMapper($this->getDataMapper())
-            ->getForm();
-        $options = array('validation_groups' => array('group1', 'group2'));
-        $form = $this->getBuilder('name', '\stdClass', $options)->getForm();
-        $parent->add($form);
-
-        $form->setData($object);
-
-        $this->expectValidateAt(0, 'data', $object, 'group1');
-        $this->expectValidateAt(1, 'data', $object, 'group2');
-
-        $this->validator->validate($form, new Form());
-
-        $this->assertNoViolation();
-    }
-
-    public function testValidateIfChildWithValidConstraint()
-    {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $parent = $this->getBuilder('parent')
             ->setCompound(true)
@@ -156,11 +123,11 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
         $this->assertNoViolation();
     }
 
-    public function testDontValidateIfParentWithoutCascadeValidation()
+    public function testDontValidateIfParentWithoutValidConstraint()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
-        $parent = $this->getBuilder('parent', null, array('cascade_validation' => false))
+        $parent = $this->getBuilder('parent', null)
             ->setCompound(true)
             ->setDataMapper($this->getDataMapper())
             ->getForm();
@@ -183,20 +150,20 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
         $form = new FormBuilder('name', '\stdClass', $this->dispatcher, $this->factory);
         $form = $form->setData($object)->getForm();
 
-        $this->expectValidateAt(0, 'data', $object, 'Default');
+        $this->expectValidateAt(0, 'data', $object, array('Default'));
 
         $this->validator->validate($form, new Form());
 
         $this->assertNoViolation();
     }
 
-    public function testValidateConstraintsEvenIfNoCascadeValidation()
+    public function testValidateConstraintsOptionEvenIfNoValidConstraint()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $constraint1 = new NotNull(array('groups' => array('group1', 'group2')));
         $constraint2 = new NotBlank(array('groups' => 'group2'));
 
-        $parent = $this->getBuilder('parent', null, array('cascade_validation' => false))
+        $parent = $this->getBuilder('parent', null)
             ->setCompound(true)
             ->setDataMapper($this->getDataMapper())
             ->getForm();
@@ -219,7 +186,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testDontValidateIfNoValidationGroups()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $form = $this->getBuilder('name', '\stdClass', array(
                 'validation_groups' => array(),
@@ -238,9 +205,9 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testDontValidateConstraintsIfNoValidationGroups()
     {
-        $object = $this->getMock('\stdClass');
-        $constraint1 = $this->getMock('Symfony\Component\Validator\Constraint');
-        $constraint2 = $this->getMock('Symfony\Component\Validator\Constraint');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
+        $constraint1 = $this->getMockBuilder('Symfony\Component\Validator\Constraint')->getMock();
+        $constraint2 = $this->getMockBuilder('Symfony\Component\Validator\Constraint')->getMock();
 
         $options = array(
             'validation_groups' => array(),
@@ -262,7 +229,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testDontValidateIfNotSynchronized()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $form = $this->getBuilder('name', '\stdClass', array(
                 'invalid_message' => 'invalid_message_key',
@@ -296,7 +263,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testAddInvalidErrorEvenIfNoValidationGroups()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $form = $this->getBuilder('name', '\stdClass', array(
                 'invalid_message' => 'invalid_message_key',
@@ -331,9 +298,9 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testDontValidateConstraintsIfNotSynchronized()
     {
-        $object = $this->getMock('\stdClass');
-        $constraint1 = $this->getMock('Symfony\Component\Validator\Constraint');
-        $constraint2 = $this->getMock('Symfony\Component\Validator\Constraint');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
+        $constraint1 = $this->getMockBuilder('Symfony\Component\Validator\Constraint')->getMock();
+        $constraint2 = $this->getMockBuilder('Symfony\Component\Validator\Constraint')->getMock();
 
         $options = array(
             'invalid_message' => 'invalid_message_key',
@@ -366,7 +333,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
     // https://github.com/symfony/symfony/issues/4359
     public function testDontMarkInvalidIfAnyChildIsNotSynchronized()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $failingTransformer = new CallbackTransformer(
             function ($data) { return $data; },
@@ -394,16 +361,31 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
         $this->assertNoViolation();
     }
 
+    public function testHandleGroupSequenceValidationGroups()
+    {
+        $object = $this->getMockBuilder('\stdClass')->getMock();
+        $options = array('validation_groups' => new GroupSequence(array('group1', 'group2')));
+        $form = $this->getBuilder('name', '\stdClass', $options)
+            ->setData($object)
+            ->getForm();
+
+        $this->expectValidateAt(0, 'data', $object, new GroupSequence(array('group1', 'group2')));
+        $this->expectValidateAt(1, 'data', $object, new GroupSequence(array('group1', 'group2')));
+
+        $this->validator->validate($form, new Form());
+
+        $this->assertNoViolation();
+    }
+
     public function testHandleCallbackValidationGroups()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $options = array('validation_groups' => array($this, 'getValidationGroups'));
         $form = $this->getBuilder('name', '\stdClass', $options)
             ->setData($object)
             ->getForm();
 
-        $this->expectValidateAt(0, 'data', $object, 'group1');
-        $this->expectValidateAt(1, 'data', $object, 'group2');
+        $this->expectValidateAt(0, 'data', $object, array('group1', 'group2'));
 
         $this->validator->validate($form, new Form());
 
@@ -412,13 +394,13 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testDontExecuteFunctionNames()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $options = array('validation_groups' => 'header');
         $form = $this->getBuilder('name', '\stdClass', $options)
             ->setData($object)
             ->getForm();
 
-        $this->expectValidateAt(0, 'data', $object, 'header');
+        $this->expectValidateAt(0, 'data', $object, array('header'));
 
         $this->validator->validate($form, new Form());
 
@@ -427,7 +409,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testHandleClosureValidationGroups()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $options = array('validation_groups' => function (FormInterface $form) {
             return array('group1', 'group2');
         });
@@ -435,8 +417,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
             ->setData($object)
             ->getForm();
 
-        $this->expectValidateAt(0, 'data', $object, 'group1');
-        $this->expectValidateAt(1, 'data', $object, 'group2');
+        $this->expectValidateAt(0, 'data', $object, array('group1', 'group2'));
 
         $this->validator->validate($form, new Form());
 
@@ -445,7 +426,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testUseValidationGroupOfClickedButton()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $parent = $this->getBuilder('parent')
             ->setCompound(true)
@@ -472,7 +453,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testDontUseValidationGroupOfUnclickedButton()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $parent = $this->getBuilder('parent')
             ->setCompound(true)
@@ -499,7 +480,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testUseInheritedValidationGroup()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $parentOptions = array('validation_groups' => 'group');
         $parent = $this->getBuilder('parent', null, $parentOptions)
@@ -521,7 +502,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testUseInheritedCallbackValidationGroup()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $parentOptions = array('validation_groups' => array($this, 'getValidationGroups'));
         $parent = $this->getBuilder('parent', null, $parentOptions)
@@ -543,7 +524,7 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testUseInheritedClosureValidationGroup()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
 
         $parentOptions = array(
             'validation_groups' => function (FormInterface $form) {
@@ -569,12 +550,12 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     public function testAppendPropertyPath()
     {
-        $object = $this->getMock('\stdClass');
+        $object = $this->getMockBuilder('\stdClass')->getMock();
         $form = $this->getBuilder('name', '\stdClass')
             ->setData($object)
             ->getForm();
 
-        $this->expectValidateAt(0, 'data', $object, 'Default');
+        $this->expectValidateAt(0, 'data', $object, array('Default'));
 
         $this->validator->validate($form, new Form());
 
@@ -631,11 +612,6 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
         $context->expects($this->never())
             ->method('addViolation');
 
-        if ($context instanceof ExecutionContextInterface) {
-            $context->expects($this->never())
-                ->method('addViolationAt');
-        }
-
         $this->validator->initialize($context);
         $this->validator->validate($form, new Form());
     }
@@ -652,9 +628,9 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
 
     private function getMockExecutionContext()
     {
-        $context = $this->getMock('Symfony\Component\Validator\Context\ExecutionContextInterface');
-        $validator = $this->getMock('Symfony\Component\Validator\Validator\ValidatorInterface');
-        $contextualValidator = $this->getMock('Symfony\Component\Validator\Validator\ContextualValidatorInterface');
+        $context = $this->getMockBuilder('Symfony\Component\Validator\Context\ExecutionContextInterface')->getMock();
+        $validator = $this->getMockBuilder('Symfony\Component\Validator\Validator\ValidatorInterface')->getMock();
+        $contextualValidator = $this->getMockBuilder('Symfony\Component\Validator\Validator\ContextualValidatorInterface')->getMock();
 
         $validator->expects($this->any())
             ->method('inContext')
@@ -702,6 +678,6 @@ class FormValidatorTest extends AbstractConstraintValidatorTest
      */
     private function getDataMapper()
     {
-        return $this->getMock('Symfony\Component\Form\DataMapperInterface');
+        return $this->getMockBuilder('Symfony\Component\Form\DataMapperInterface')->getMock();
     }
 }
